@@ -1,14 +1,16 @@
+from __future__ import annotations
+
 import asyncio
 import importlib.util
 import inspect
 import json
-import os
 import time
 from abc import ABC, abstractmethod
 from collections.abc import AsyncIterator, Callable, Iterator
 from functools import wraps
-from typing import TYPE_CHECKING, Any, Hashable, Mapping, Optional, Union
+from typing import TYPE_CHECKING, Any, Hashable, Mapping, MutableMapping, Optional, Union
 
+import wrapt
 from openinference.instrumentation import safe_json_dumps
 from openinference.semconv.trace import (
     OpenInferenceLLMProviderValues,
@@ -19,6 +21,7 @@ from strawberry import UNSET
 from strawberry.scalars import JSON as JSONScalarType
 from typing_extensions import TypeAlias, assert_never
 
+from phoenix.config import getenv
 from phoenix.evals.models.rate_limiters import (
     AsyncCallable,
     GenericType,
@@ -51,11 +54,13 @@ from phoenix.server.api.types.ChatCompletionSubscriptionPayload import (
 from phoenix.server.api.types.GenerativeProvider import GenerativeProviderKey
 
 if TYPE_CHECKING:
+    import httpx
     from anthropic.types import MessageParam, TextBlockParam, ToolResultBlockParam
     from google.generativeai.types import ContentType
     from openai import AsyncAzureOpenAI, AsyncOpenAI
     from openai.types import CompletionUsage
     from openai.types.chat import ChatCompletionMessageParam, ChatCompletionMessageToolCallParam
+    from opentelemetry.util.types import AttributeValue
 
 SetSpanAttributesFn: TypeAlias = Callable[[Mapping[str, Any]], None]
 ChatCompletionChunk: TypeAlias = Union[TextChunk, ToolCallChunk]
@@ -169,7 +174,7 @@ class PlaygroundStreamingClient(ABC):
         model: GenerativeModelInput,
         api_key: Optional[str] = None,
     ) -> None:
-        self._attributes: dict[str, Any] = dict()
+        self._attributes: dict[str, AttributeValue] = dict()
 
     @classmethod
     @abstractmethod
@@ -246,6 +251,7 @@ class OpenAIBaseStreamingClient(PlaygroundStreamingClient):
         self.client = client
         self.model_name = model.name
         self.rate_limiter = PlaygroundRateLimiter(model.provider_key, OpenAIRateLimitError)
+        self.client._client = _HttpxClient(self.client._client, self._attributes)
 
     @classmethod
     def dependencies(cls) -> list[Dependency]:
@@ -452,6 +458,7 @@ class OpenAIBaseStreamingClient(PlaygroundStreamingClient):
     model_names=[
         PROVIDER_DEFAULT,
         "gpt-4o",
+        "gpt-4o-2024-11-20",
         "gpt-4o-2024-08-06",
         "gpt-4o-2024-05-13",
         "chatgpt-4o-latest",
@@ -467,6 +474,8 @@ class OpenAIBaseStreamingClient(PlaygroundStreamingClient):
         "gpt-3.5-turbo-0125",
         "gpt-3.5-turbo",
         "gpt-3.5-turbo-1106",
+        # preview models
+        "gpt-4.5-preview",
     ],
 )
 class OpenAIStreamingClient(OpenAIBaseStreamingClient):
@@ -477,8 +486,8 @@ class OpenAIStreamingClient(OpenAIBaseStreamingClient):
     ) -> None:
         from openai import AsyncOpenAI
 
-        base_url = model.base_url or os.environ.get("OPENAI_BASE_URL")
-        if not (api_key := api_key or os.environ.get("OPENAI_API_KEY")):
+        base_url = model.base_url or getenv("OPENAI_BASE_URL")
+        if not (api_key := api_key or getenv("OPENAI_API_KEY")):
             if not base_url:
                 raise BadRequest("An API key is required for OpenAI models")
             api_key = "sk-fake-api-key"
@@ -650,17 +659,33 @@ class AzureOpenAIStreamingClient(OpenAIBaseStreamingClient):
     ):
         from openai import AsyncAzureOpenAI
 
-        if not (api_key := api_key or os.environ.get("AZURE_OPENAI_API_KEY")):
-            raise BadRequest("An Azure API key is required for Azure OpenAI models")
-        if not (endpoint := model.endpoint or os.environ.get("AZURE_OPENAI_ENDPOINT")):
+        if not (endpoint := model.endpoint or getenv("AZURE_OPENAI_ENDPOINT")):
             raise BadRequest("An Azure endpoint is required for Azure OpenAI models")
-        if not (api_version := model.api_version or os.environ.get("OPENAI_API_VERSION")):
+        if not (api_version := model.api_version or getenv("OPENAI_API_VERSION")):
             raise BadRequest("An OpenAI API version is required for Azure OpenAI models")
-        client = AsyncAzureOpenAI(
-            api_key=api_key,
-            azure_endpoint=endpoint,
-            api_version=api_version,
-        )
+        if api_key := api_key or getenv("AZURE_OPENAI_API_KEY"):
+            client = AsyncAzureOpenAI(
+                api_key=api_key,
+                azure_endpoint=endpoint,
+                api_version=api_version,
+            )
+        else:
+            try:
+                from azure.identity.aio import DefaultAzureCredential, get_bearer_token_provider
+            except ImportError:
+                raise BadRequest(
+                    "Provide an API key for Azure OpenAI models or use azure-identity, see. e.g. "
+                    "https://learn.microsoft.com/en-us/python/api/azure-identity/azure.identity.environmentcredential?view=azure-python"  # noqa: E501
+                )
+
+            client = AsyncAzureOpenAI(
+                azure_ad_token_provider=get_bearer_token_provider(
+                    DefaultAzureCredential(),
+                    "https://cognitiveservices.azure.com/.default",
+                ),
+                azure_endpoint=endpoint,
+                api_version=api_version,
+            )
         super().__init__(client=client, model=model, api_key=api_key)
         self._attributes[LLM_PROVIDER] = OpenInferenceLLMProviderValues.AZURE.value
         self._attributes[LLM_SYSTEM] = OpenInferenceLLMSystemValues.OPENAI.value
@@ -691,11 +716,12 @@ class AnthropicStreamingClient(PlaygroundStreamingClient):
         super().__init__(model=model, api_key=api_key)
         self._attributes[LLM_PROVIDER] = OpenInferenceLLMProviderValues.ANTHROPIC.value
         self._attributes[LLM_SYSTEM] = OpenInferenceLLMSystemValues.ANTHROPIC.value
-        if not (api_key := api_key or os.environ.get("ANTHROPIC_API_KEY")):
+        if not (api_key := api_key or getenv("ANTHROPIC_API_KEY")):
             raise BadRequest("An API key is required for Anthropic models")
         self.client = anthropic.AsyncAnthropic(api_key=api_key)
         self.model_name = model.name
         self.rate_limiter = PlaygroundRateLimiter(model.provider_key, anthropic.RateLimitError)
+        self.client._client = _HttpxClient(self.client._client, self._attributes)
 
     @classmethod
     def dependencies(cls) -> list[Dependency]:
@@ -799,6 +825,10 @@ class AnthropicStreamingClient(PlaygroundStreamingClient):
                     raise NotImplementedError
                 elif isinstance(event, anthropic_streaming._types.CitationEvent):
                     raise NotImplementedError
+                elif isinstance(event, anthropic_streaming._types.ThinkingEvent):
+                    pass
+                elif isinstance(event, anthropic_streaming._types.SignatureEvent):
+                    pass
                 else:
                     assert_never(event)
 
@@ -849,7 +879,28 @@ class AnthropicStreamingClient(PlaygroundStreamingClient):
 
 
 @register_llm_client(
-    provider_key=GenerativeProviderKey.GEMINI,
+    provider_key=GenerativeProviderKey.ANTHROPIC,
+    model_names=[
+        "claude-3-7-sonnet-latest",
+        "claude-3-7-sonnet-20250219",
+    ],
+)
+class AnthropicReasoningStreamingClient(AnthropicStreamingClient):
+    @classmethod
+    def supported_invocation_parameters(cls) -> list[InvocationParameter]:
+        invocation_params = super().supported_invocation_parameters()
+        invocation_params.append(
+            JSONInvocationParameter(
+                invocation_name="thinking",
+                canonical_name=CanonicalParameterName.ANTHROPIC_EXTENDED_THINKING,
+                label="Thinking Budget",
+            )
+        )
+        return invocation_params
+
+
+@register_llm_client(
+    provider_key=GenerativeProviderKey.GOOGLE,
     model_names=[
         PROVIDER_DEFAULT,
         "gemini-2.0-flash-exp",
@@ -859,7 +910,7 @@ class AnthropicStreamingClient(PlaygroundStreamingClient):
         "gemini-1.0-pro",
     ],
 )
-class GeminiStreamingClient(PlaygroundStreamingClient):
+class GoogleStreamingClient(PlaygroundStreamingClient):
     def __init__(
         self,
         model: GenerativeModelInput,
@@ -870,11 +921,7 @@ class GeminiStreamingClient(PlaygroundStreamingClient):
         super().__init__(model=model, api_key=api_key)
         self._attributes[LLM_PROVIDER] = OpenInferenceLLMProviderValues.GOOGLE.value
         self._attributes[LLM_SYSTEM] = OpenInferenceLLMSystemValues.VERTEXAI.value
-        if not (
-            api_key := api_key
-            or os.environ.get("GEMINI_API_KEY")
-            or os.environ.get("GOOGLE_API_KEY")
-        ):
+        if not (api_key := api_key or getenv("GEMINI_API_KEY") or getenv("GOOGLE_API_KEY")):
             raise BadRequest("An API key is required for Gemini models")
         google_genai.configure(api_key=api_key)
         self.model_name = model.name
@@ -938,7 +985,7 @@ class GeminiStreamingClient(PlaygroundStreamingClient):
     ) -> AsyncIterator[ChatCompletionChunk]:
         import google.generativeai as google_genai
 
-        gemini_message_history, current_message, system_prompt = self._build_gemini_messages(
+        google_message_history, current_message, system_prompt = self._build_google_messages(
             messages
         )
 
@@ -947,17 +994,17 @@ class GeminiStreamingClient(PlaygroundStreamingClient):
             model_args["system_instruction"] = system_prompt
         client = google_genai.GenerativeModel(**model_args)
 
-        gemini_config = google_genai.GenerationConfig(
+        google_config = google_genai.GenerationConfig(
             **invocation_parameters,
         )
-        gemini_params = {
+        google_params = {
             "content": current_message,
-            "generation_config": gemini_config,
+            "generation_config": google_config,
             "stream": True,
         }
 
-        chat = client.start_chat(history=gemini_message_history)
-        stream = await chat.send_message_async(**gemini_params)
+        chat = client.start_chat(history=google_message_history)
+        stream = await chat.send_message_async(**google_params)
         async for event in stream:
             self._attributes.update(
                 {
@@ -968,29 +1015,29 @@ class GeminiStreamingClient(PlaygroundStreamingClient):
             )
             yield TextChunk(content=event.text)
 
-    def _build_gemini_messages(
+    def _build_google_messages(
         self,
         messages: list[tuple[ChatCompletionMessageRole, str, Optional[str], Optional[list[str]]]],
     ) -> tuple[list["ContentType"], str, str]:
-        gemini_message_history: list["ContentType"] = []
+        google_message_history: list["ContentType"] = []
         system_prompts = []
         for role, content, _tool_call_id, _tool_calls in messages:
             if role == ChatCompletionMessageRole.USER:
-                gemini_message_history.append({"role": "user", "parts": content})
+                google_message_history.append({"role": "user", "parts": content})
             elif role == ChatCompletionMessageRole.AI:
-                gemini_message_history.append({"role": "model", "parts": content})
+                google_message_history.append({"role": "model", "parts": content})
             elif role == ChatCompletionMessageRole.SYSTEM:
                 system_prompts.append(content)
             elif role == ChatCompletionMessageRole.TOOL:
                 raise NotImplementedError
             else:
                 assert_never(role)
-        if gemini_message_history:
-            prompt = gemini_message_history.pop()["parts"]
+        if google_message_history:
+            prompt = google_message_history.pop()["parts"]
         else:
             prompt = ""
 
-        return gemini_message_history, prompt, "\n".join(system_prompts)
+        return google_message_history, prompt, "\n".join(system_prompts)
 
 
 def initialize_playground_clients() -> None:
@@ -1005,3 +1052,15 @@ LLM_SYSTEM = SpanAttributes.LLM_SYSTEM
 LLM_TOKEN_COUNT_PROMPT = SpanAttributes.LLM_TOKEN_COUNT_PROMPT
 LLM_TOKEN_COUNT_COMPLETION = SpanAttributes.LLM_TOKEN_COUNT_COMPLETION
 LLM_TOKEN_COUNT_TOTAL = SpanAttributes.LLM_TOKEN_COUNT_TOTAL
+
+
+class _HttpxClient(wrapt.ObjectProxy):  # type: ignore
+    def __init__(self, wrapped: httpx.AsyncClient, attributes: MutableMapping[str, Any]):
+        super().__init__(wrapped)
+        self._self_attributes = attributes
+
+    async def send(self, request: httpx.Request, **kwargs: Any) -> Any:
+        self._self_attributes["url.full"] = str(request.url)
+        self._self_attributes["url.path"] = request.url.path.removeprefix(self.base_url.path)
+        response = await self.__wrapped__.send(request, **kwargs)
+        return response
